/*!
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/.
 */

import { Schema as S } from "@effect/schema";
import type { MimeTypePredicate } from "./MimeType";
import type { Multiplicity } from "./Multiplicity";

// --- INTERFACES --------------------------------------------------------------

export interface FieldText<M extends Multiplicity, Schema extends S.Schema<any, string, unknown>> {
	readonly _tag: "Text";
	readonly multiplicity: M;
	readonly schema: Schema;
}

export interface FieldJson<M extends Multiplicity, Schema extends S.Schema.Any> {
	readonly _tag: "Json";
	readonly multiplicity: M;
	readonly schema: Schema;
}

export interface FieldFile<M extends Multiplicity, MimeType extends string> {
	readonly _tag: "File";
	readonly multiplicity: M;
	readonly mimeType: MimeTypePredicate<MimeType>;
}

export namespace FieldText {
	export type Any = FieldText<Multiplicity, S.Schema<any, string, unknown>>;
}

export namespace FieldJson {
	export type Any = FieldJson<Multiplicity, S.Schema.Any>;
}

export namespace FieldFile {
	export type Any = FieldFile<Multiplicity, string>;
}

export namespace Field {
	export type Any =
		| FieldText.Any
		| FieldJson.Any
		| FieldFile.Any
		;

	export type AnyUrlEncoded =
		| FieldText.Any
		| FieldJson.Any
		;
}

// --- CONSTRUCTORS ------------------------------------------------------------

export const FieldText = <const M extends Multiplicity, const Schema extends S.Schema<any, string, unknown>>(multiplicity: M, schema: Schema): FieldText<M, Schema> => Object.freeze<FieldText<M, Schema>>({
	_tag: "Text",
	multiplicity,
	schema,
});

export const FieldJson = <const M extends Multiplicity, const Schema extends S.Schema.Any>(multiplicity: M, schema: Schema): FieldJson<M, Schema> => Object.freeze<FieldJson<M, Schema>>({
	_tag: "Json",
	multiplicity,
	schema,
});

export const FieldFile = <const M extends Multiplicity, const MimeType extends string>(multiplicity: M, mimeType: MimeTypePredicate<MimeType>): FieldFile<M, MimeType> => Object.freeze<FieldFile<M, MimeType>>({
	_tag: "File",
	multiplicity,
	mimeType,
});
