/*!
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at http://mozilla.org/MPL/2.0/.
 */

import { Schema as S } from "@effect/schema";

// --- INTERFACES --------------------------------------------------------------

export interface RouteLiteral<Literal extends string> {
	readonly _tag: "Literal";
	readonly literal: Literal;
}

export interface RouteParam<Name extends string, Schema extends S.Schema<any, string, unknown>> {
	readonly _tag: "Param";
	readonly name: Name;
	readonly schema: Schema;
}

export namespace RouteLiteral {
	export type Any = RouteLiteral<string>;
}

export namespace RouteParam {
	export type Any = RouteParam<string, S.Schema<any, string, unknown>>;
}

export namespace Route {
	export type Any = readonly (
		| RouteLiteral.Any
		| RouteParam.Any
	)[];
}

// --- CONSTRUCTORS ------------------------------------------------------------

export const RouteLiteral = <const Literal extends string>(literal: Literal): RouteLiteral<Literal> => Object.freeze<RouteLiteral<Literal>>({
	_tag: "Literal",
	literal,
});

export const RouteParam = <const Name extends string, const Schema extends S.Schema<any, string, unknown>>(name: Name, schema: Schema): RouteParam<Name, Schema> => Object.freeze<RouteParam<Name, Schema>>({
	_tag: "Param",
	name,
	schema,
});

export const Route = <const Route extends Route.Any>(...route: Route): Route => Object.freeze<Route>(route);
